/* arch/arm/boot/compressed/tomtomgo-init.c
 *
 * Special initialization for TomTom GO.
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 * Author: Dimitry Andric <dimitry.andric@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <asm/arch/tomtomgo-init.h>

#include <asm/io.h>
#include <asm/arch/map.h>

/* We need to define these locally to prevent mistakes */
#define S3C2410_GSTATUS1	(S3C2410_PA_GPIO   + 0xB0)
#define S3C2440_DSC0		(S3C2410_PA_GPIO   + 0xC4)
#define S3C2440_DSC1		(S3C2410_PA_GPIO   + 0xC8)
#define S3C2410_CAMDIVN     (S3C2410_PA_CLKPWR + 0x18)
#define S3C2410_CLKDIVN		(S3C2410_PA_CLKPWR + 0x14)
#define S3C2410_MPLLCON		(S3C2410_PA_CLKPWR + 0x04)
#define S3C2410_UPLLCON		(S3C2410_PA_CLKPWR + 0x08)
#define S3C2410_ULCON0 		(S3C2410_PA_UART   + 0x00)
#define S3C2410_UCON0  		(S3C2410_PA_UART   + 0x04)
#define S3C2410_UFCON0 		(S3C2410_PA_UART   + 0x08)
#define S3C2410_UMCON0 		(S3C2410_PA_UART   + 0x0C)
#define S3C2410_UBRDIV0		(S3C2410_PA_UART   + 0x28)

#define S3C2410_GSTATUS1_IDMASK	   (0xffff0000)
#define S3C2410_GSTATUS1_2410	   (0x32410000)
#define S3C2410_GSTATUS1_2440	   (0x32440000)
#define S3C2410_GSTATUS1_2442      (0x32440aaa)

/* Chose baud rate and divisor calculation */
#define CURRENT_BAUD_RATE	115200
#define UBRDIV(pclk, baud)	(pclk / 16 / baud - 1)

/* Chosen CPU speeds for different models */
#define S3C2410_PLLSPEED 199200000
#ifdef CONFIG_SMDK2440_BOARD
#define S3C2440_PLLSPEED 399651840
#else
#define S3C2440_PLLSPEED 376800000 // 376800000 // 393000000 // 376800000 // 405600000
#endif

/* Dividers for S3C2410 */
#if S3C2410_PLLSPEED == 199200000
#define S3C2410_HDIVN	1
#define S3C2410_PDIVN	1
#define S3C2410_MDIV	0x9e
#define S3C2410_PDIV	3
#define S3C2410_SDIV	1
#define S3C2410_HCLKDIV	2
#define S3C2410_PCLKDIV	4
#elif S3C2410_PLLSPEED == 202000000
#define S3C2410_HDIVN	1
#define S3C2410_PDIVN	1
#define S3C2410_MDIV	0x5d
#define S3C2410_PDIV	1
#define S3C2410_SDIV	1
#define S3C2410_HCLKDIV	2
#define S3C2410_PCLKDIV	4
#elif S3C2410_PLLSPEED == 202800000
#define S3C2410_HDIVN	1
#define S3C2410_PDIVN	1
#define S3C2410_MDIV	0xa1
#define S3C2410_PDIV	3
#define S3C2410_SDIV	1
#define S3C2410_HCLKDIV	2
#define S3C2410_PCLKDIV	4
#else
#error "Select a proper S3C2410 PLL speed"
#endif

#define S3C2410_PCLK	(S3C2410_PLLSPEED / S3C2410_PCLKDIV)

/* Dividers for S3C2440 */
#if S3C2440_PLLSPEED == 399651840
#define S3C2440_HDIVN	2
#define S3C2440_PDIVN	1
#define S3C2440_MDIV	0x6e
#define S3C2440_PDIV	3
#define S3C2440_SDIV	1
#define S3C2410_HCLKDIV	4
#define S3C2410_PCLKDIV	8

// a1h, 3h,0h |405600000|405M|  2.4M| 0.1| 0.1| 0.2|0|
#elif S3C2440_PLLSPEED == 405600000
#define S3C2440_HDIVN	3
#define S3C2440_PDIVN	1
#define S3C2440_MDIV	0xa1
#define S3C2440_PDIV	3
#define S3C2440_SDIV	1
#define S3C2440_HCLKDIV	3
#define S3C2440_PCLKDIV	6

// 95h, 3h,0h |376800000|376M|  2.4M| 0.1| 0.1| 0.2|0|
#elif S3C2440_PLLSPEED == 376800000
#define S3C2440_HDIVN	2
#define S3C2440_PDIVN	1
#define S3C2440_MDIV	0x95
#define S3C2440_PDIV	3
#define S3C2440_SDIV	1
#define S3C2442_PDIV	8
#define S3C2442_SDIV	0
#define S3C2440_HCLKDIV	4
#define S3C2440_PCLKDIV	8

// 95h, 3h,0h |376800000|376M|  2.4M| 0.1| 0.1| 0.2|0|
#elif S3C2440_PLLSPEED == 400000000
#define S3C2440_HDIVN	2
#define S3C2440_PDIVN	1
#define S3C2440_MDIV	42
#define S3C2440_PDIV	1
#define S3C2440_SDIV	1
#define S3C2440_HCLKDIV	4
#define S3C2440_PCLKDIV	8

// 7bh, 2h,0h |393000000|393M|  3.0M| 0.1| 0.1| 0.3|0|
#elif S3C2440_PLLSPEED == 393000000
#define S3C2440_HDIVN	2
#define S3C2440_PDIVN	1
#define S3C2440_MDIV	0x7b
#define S3C2440_PDIV	2
#define S3C2440_SDIV	1
#define S3C2440_HCLKDIV	4
#define S3C2440_PCLKDIV	8

// 9ch, 3h,0h |393600000|393M|  2.4M| 0.1| 0.1| 0.2|0|
#elif S3C2440_PLLSPEED == 393000000
#define S3C2440_HDIVN	2
#define S3C2440_PDIVN	1
#define S3C2440_MDIV	0x9c
#define S3C2440_PDIV	3
#define S3C2440_SDIV	1
#define S3C2440_HCLKDIV	4
#define S3C2440_PCLKDIV	8

// 78h, 3h,0h |307200000|307M|  2.4M| 0.1| 0.1| 0.2|0|
#elif S3C2440_PLLSPEED == 307200000
#define S3C2440_HDIVN	3
#define S3C2440_PDIVN	1
#define S3C2440_MDIV	0x78
#define S3C2440_PDIV	3
#define S3C2440_SDIV	1
#define S3C2440_HCLKDIV	3
#define S3C2440_PCLKDIV	6

#else
#error "Select a proper S3C2440 PLL speed"
#endif

#define S3C2440_PCLK	(S3C2440_PLLSPEED / S3C2440_PCLKDIV)

void tomtomgo_init(void)
{
	unsigned long cpu;
	unsigned long clkdivn;
	unsigned long mpllcon;
	unsigned long ubrdiv;
	int i;

	 /* USB clock 48 Mhz, like bootloader does it*/
	__raw_writel(0x38022, S3C2410_UPLLCON);

	/* Determine CPU type */
	cpu = __raw_readl(S3C2410_GSTATUS1);

	switch (cpu & S3C2410_GSTATUS1_IDMASK) {
	case S3C2410_GSTATUS1_2410:
		/* S3C2410 */
		clkdivn = (S3C2410_HDIVN<<1) | S3C2410_PDIVN;
		mpllcon = (S3C2410_MDIV<<12) | (S3C2410_PDIV<<4) | S3C2410_SDIV;
		ubrdiv = UBRDIV(S3C2410_PCLK, CURRENT_BAUD_RATE);
		break;
	case S3C2410_GSTATUS1_2440:
		/* S3C2440 */
		clkdivn = (S3C2440_HDIVN<<1) | S3C2440_PDIVN;
		ubrdiv = UBRDIV(S3C2440_PCLK, CURRENT_BAUD_RATE);
		if (cpu == S3C2410_GSTATUS1_2442) {
			mpllcon = (S3C2440_MDIV<<12) | (S3C2442_PDIV<<4) | S3C2442_SDIV;
			/* don't reduce the bus driver power because of 1.8 volt logic */
			__raw_writel(0x00000000, S3C2440_DSC0);
			__raw_writel(0x00000000, S3C2440_DSC1);
		} else {
			mpllcon = (S3C2440_MDIV<<12) | (S3C2440_PDIV<<4) | S3C2440_SDIV;
			/* reduce the bus driver power to lower the noise floor on S3C2440 */
			__raw_writel(0x7fffffff, S3C2440_DSC0);
			__raw_writel(0xffffffff, S3C2440_DSC1);
		}
		break;
	default:
		/* Unknown CPU, just don't touch anything */
		return;
	}

	/* Setup all registers */
	__raw_writel(clkdivn, S3C2410_CLKDIVN); /* Clock divider */
	__raw_writel(mpllcon, S3C2410_MPLLCON); /* MPLL configuration */
	__raw_writel(__raw_readl(S3C2410_CAMDIVN) & ~(3<<8), S3C2410_CAMDIVN); /* Clock divider */
	__raw_writel(0x0000,  S3C2410_UFCON0);  /* FIFO control register, FIFO disable */
	__raw_writel(0x0000,  S3C2410_UMCON0);  /* Modem control register, AFC disable */
	__raw_writel(0x0003,  S3C2410_ULCON0);  /* Line control register, 8N1 */
	__raw_writel(0x0245,  S3C2410_UCON0);   /* Control register */
	__raw_writel(ubrdiv,  S3C2410_UBRDIV0); /* Baud rate divisor register */

	/* Delay for a little while */
	for (i = 0; i < 100; ++i)
		;
}

/* EOF */

