/* include/barcelona/Barc_Gpio.h
 *
 * Public interface for the GPIO driver.
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 * Author: Dimitry Andric <dimitry.andric@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __INCLUDE_BARCELONA_BARC_GPIO_H
#define __INCLUDE_BARCELONA_BARC_GPIO_H

#ifndef __INCLUDE_BARCELONA_TYPES_H
#include <barcelona/types.h>
#endif /* __INCLUDE_BARCELONA_TYPES_H */

#include <linux/ioctl.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define GPIO_DEVNAME					"hwstatus"
#define GPIO_MAJOR						240

/* For IOR_HWSTATUS ioctl */
typedef struct {
	UINT8 u8ChargeStatus;
	UINT8 u8InputStatus;
	UINT8 u8DockStatus;
#if 0
	UINT8 u8PowerButtonStatus;
	UINT8 u8UsbStatus;
	UINT8 u8LightsStatus;
	UINT8 u8IgnitionStatus;
	UINT8 u8ExtMicStatus;
	UINT8 u8HeadphoneStatus;
	UINT8 u8SerialStatus;
#endif
} HARDWARE_STATUS;

/* For HARDWARE_STATUS::u8ChargeStatus */
#define CHARGE_STATE_ACPWR_OFF			0		/* AC power is off   */
#define CHARGE_STATE_FAULT				1		/* Charge fault      */
#define CHARGE_STATE_COMPLETE			2		/* Charging complete */
#define CHARGE_STATE_FAST_CHARGING		3		/* Fast charging     */
#define CHARGE_STATE_TOPPINGUP			4		/* Topping up        */

/* For HARDWARE_STATUS::u8InputStatus */
#define ONOFF_MASK                   0x01	/* On-off button      */
#define USB_DETECT_MASK              0x02	/* USB power detect   */
#define DOCK_LIGHTS_MASK             0x04	/* Lights detect */
#define DOCK_IGNITION_MASK           0x08	/* Ignition detect */
#define DOCK_EXTMIC_MASK             0x10	/* External microphone detect */
#define DOCK_HEADPHONE_MASK          0x20	/* Headphone detect */
#define DOCK_LINEIN_MASK             0x40	/* Line in detect PIN on K4 dock conn */
#define DOCK_RXD_DETECT_MASK       	 0x80	/* Serial detect via RXD break / interrupt signalling */

/* For HARDWARE_STATUS::u8DockStatus */
#define DOCK_NONE		0
#define DOCK_WINDSCREEN	1
#define DOCK_PERMANENT	2
#define DOCK_DESK		3

/* GPIO driver ioctls */
#define GPIO_DRIVER_MAGIC				'U'
#define IOR_HWSTATUS					_IOR(GPIO_DRIVER_MAGIC, 0, HARDWARE_STATUS)
#define IOR_DISK_ACCESS       _IOR(GPIO_DRIVER_MAGIC, 7, UINT32)
#define IOR_BT_MODE			  _IOR(GPIO_DRIVER_MAGIC, 8, UINT32)
#define IOR_GET_BT_ERROR          _IOW(GPIO_DRIVER_MAGIC, 9, UINT32)
#define IOR_SET_BT_ERROR          _IOR(GPIO_DRIVER_MAGIC, 10, UINT32)

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __INCLUDE_BARCELONA_BARC_GPIO_H */

/* EOF */
