/* include/barcelona/gotype.h
 *
 * Declarations for different TomTom GO types.
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 * Authors: Dimitry Andric <dimitry.andric@tomtom.com>
 *          Jeroen Taverne <jeroen.taverne@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __INCLUDE_BARCELONA_GOTYPE_H
#define __INCLUDE_BARCELONA_GOTYPE_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* GO id pin bits */
#define GOTYPE_ID0_LO			0
#define GOTYPE_ID0_HI			1
#define GOTYPE_ID1_LO			0
#define GOTYPE_ID1_HI			2
#define GOTYPE_ID2_LO			0
#define GOTYPE_ID2_HI			8
#define GOTYPE_ID3_LO			0 
#define GOTYPE_ID3_HI			32 
#define GOTYPE_ID4_LO			0 
#define GOTYPE_ID4_HI			64 

/* GO CPU type bits */
#define GOTYPE_S3C2442			128
#define GOTYPE_S3C2440			0
#define GOTYPE_S3C2410			4

/* GO screen type bits */
#define GOTYPE_NEC_TFT			0
#define GOTYPE_SAMSUNG_TFT		16

/* GO Valencia harddisk type bits */
#define GOTYPE_VALENCIA_SDCARD		0
#define GOTYPE_VALENCIA_HARDDISK	256

/* Combined GO type identifiers */
#define GOTYPE_CLASSIC			(                GOTYPE_ID1_HI | GOTYPE_ID0_HI | GOTYPE_S3C2410 | GOTYPE_NEC_TFT)     // Should get 7
#define GOTYPE_M100				(                GOTYPE_ID1_LO | GOTYPE_ID0_HI | GOTYPE_S3C2410 | GOTYPE_NEC_TFT)     // Should get 5
#define GOTYPE_M300				(                GOTYPE_ID1_HI | GOTYPE_ID0_HI | GOTYPE_S3C2440 | GOTYPE_NEC_TFT)     // Should get 3
#define GOTYPE_M500				(                GOTYPE_ID1_HI | GOTYPE_ID0_LO | GOTYPE_S3C2440 | GOTYPE_NEC_TFT)     // Should get 2
#define GOTYPE_ATLANTA			(GOTYPE_ID2_HI | GOTYPE_ID1_LO | GOTYPE_ID0_HI | GOTYPE_S3C2440 | GOTYPE_SAMSUNG_TFT)
#define GOTYPE_ABERDEEN			(GOTYPE_ID2_HI | GOTYPE_ID1_HI | GOTYPE_ID0_HI | GOTYPE_S3C2440 | GOTYPE_SAMSUNG_TFT)
#define GOTYPE_VALENCIA			(                GOTYPE_ID1_LO | GOTYPE_ID0_LO | GOTYPE_S3C2442 | GOTYPE_SAMSUNG_TFT) // Should get 144

/* Atlanta type identifiers */
#define GOTYPE_ATLANTA_BILBAO	(GOTYPE_ATLANTA | GOTYPE_ID4_HI | GOTYPE_ID3_HI) // Should get 121
#define GOTYPE_ATLANTA_GLASGOW	(GOTYPE_ATLANTA | GOTYPE_ID4_LO | GOTYPE_ID3_HI) // Should get 57

/* Valencia type identifiers */
#define GOTYPE_VALENCIA500	(GOTYPE_VALENCIA | GOTYPE_VALENCIA_SDCARD)   // Should get 144
#define GOTYPE_VALENCIA700	(GOTYPE_VALENCIA | GOTYPE_VALENCIA_HARDDISK) // Should get 400

/* For compatibility */
#define GOTYPE_BILBAO	GOTYPE_ATLANTA_BILBAO	
#define GOTYPE_GLASGOW	GOTYPE_ATLANTA_GLASGOW	

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __INCLUDE_BARCELONA_GOTYPE_H */

/* EOF */
